#!/bin/sh

. /etc/default/globals

if dpkg -l | grep imdb-cover | grep -c ^ii 
then
	/usr/sbin/IMDB-cover-search.pl "$1"
	if [ $? -eq 0 ]
	then
		exit 0
	fi
fi

# go to recordings directory
cd "$1"

PREVIEW_IMG=preview_vdr.png

if [ -e $PREVIEW_IMG ]
then
  tl "Preview already exists. Doing nothing."
  return # preview already exists; do nothing
fi


if [ -e 00001.ts ]
then
  FILE=00001.ts
  ### count files ###
  FILECOUNT=`ls -1 *.ts | wc -l`
  ### get middle of ts-files ###
  MIDDLE=`echo scale=0\;"$FILECOUNT / 2 + 1" | bc`
  FILE=`printf '%05d' $MIDDLE`
  FILE=`echo $FILE.ts`
else
  msg=`tl "no recording found in %s. Exiting" | sed s/%s/$1/`
  smesg "$msg"
  return # no recording found
fi
if [ $MIDDLE -eq 1 ]
then
  ### if recording has only one File, then get the middle of the selected file ###
  DURATION=`ffmpeg -i $FILE 2>&1 | grep "Duration" | cut -d ' ' -f 4 | sed s/,//`
  HOURS=`echo $DURATION | cut -d ":" -f1`
  MINS=`echo $DURATION | cut -d ":" -f2`
  SECS=`echo $DURATION | cut -d ":" -f1 | cut -d "." -f1`
  DURATION_SEC=`echo "($HOURS * 3600 + $MINS * 60 + $SECS) / 2" | bc`
  MSG="${DURATION_SEC}s in File $MIDDLE"
else
  DURATION_SEC=1
  MSG="begin of File $MIDDLE"
fi

## TODO
## Generate preview intelligently (should be useful)
## usually recordings start 3-5 minute before the actual program
##    so the first frame does not represent the program's preview
## Could jump to the middle of the recording
##    that could mean, looking into each of 000xx.ts files to
##    decide which file has the middle frame
##    Even the middle frame might be of a commercial!!
##    Run NoAd and find a non-commercial "middle" frame

## preview _must_ by 126x128;
# as skin can crash if smaller or show a buggy image if larger
# -vframes 1 : one video frame (in our case, first intra frame)

PREVIEW_SIZE=126x128

## for the timebeing just look at the first intra frame for preview
ffmpeg -ss $DURATION_SEC -i "$FILE" -vframes 1 -vcodec png -an -f rawvideo -s $PREVIEW_SIZE $PREVIEW_IMG && tl "Preview generated" || tl "Error generating preview"
